/*
Snell-Descartes

Copyright (C) 2012 Observatoire de Paris-Meudon
 
Auteur : Benjamin Mollier
Conversion en Javascript : Damien Guillaume

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

/**
 * Algorithme de simulation pour SimuApplet.
 * Simule les rayons réflechis et réfractés, issus d'un rayon incident sur un dioptre plan.
 */

new SnellDescartes();

//----- I. Constructeur de la classe, avec la définition des paramètres de simulation et des options d'affichage.
function SnellDescartes() {
    //----- I. Variable globales
        // - - - I.1 Coordonnées des rayons incidents et réfléchis (x1,y1) et réfracté (x2,y2) - - - -
    this.x1 = 0.;
    this.y1 = 0.;
    this.x2 = 0.;
    this.y2 = 0.;
    this.s = 0.;
    
        // - - - I.2 Pas pour tracés les rayons - - - 
    this.ds = 0.1;
    
        // - - - I.3 Angles réfléchi et réfracté - - -
    this.r1 = 0.;
    this.i2 = 0.;
    this.i1 = 0.;
    this.n1 = 0.;
    this.n2 = 0.;
    
        // - - - I.4 Options graphiques - - -
    this.s1 = '';
    this.s2 = '';
    
    this.applet = new SimuApplet(this);
    this.applet.appletDoc('snell_descartes_doc.xml');
    this.applet.setDelaiIterations(Math.round(this.ds * 100));
}

	//----- II. Calcul des rayons
		// - - - II.1 Calcul live (utilisé avec affichage "plotlive") - - - - 
			// - 2.1.1 Initialisation -
SnellDescartes.prototype.initCalculLive = function(lin) {
    // - 3.1.1.1 Chargement des variables d'entrée -
    this.i1 = lin.lireDouble('i1');
    this.n1 = (Math.abs(this.i1) <= 90 ? lin.lireDouble('n1') : lin.lireDouble('n2'));
    this.n2 = (Math.abs(this.i1) <= 90 ? lin.lireDouble('n2') : lin.lireDouble('n1'));
    this.s1 = lin.lireString('s1');
    this.s2 = lin.lireString('s2');


    // - 3.1.1.2 Initialisation des coordonnées -
    this.s  =  8;
    this.x1 = -this.s * Math.sin(this.i1 / 180. * Math.PI);
    this.y1 =  this.s * Math.cos(this.i1 / 180. * Math.PI);

    // - 3.1.1.3 Calcul des angles réfléchi et réfracté
    this.r1 = -this.i1;
    this.i2 = 180 / Math.PI * Math.asin(this.n1 / this.n2 * Math.sin(this.i1 / 180 * Math.PI));
}

// - 3.1.2 Calcul de l'abscisse curviligne s à s+ds
SnellDescartes.prototype.calculLive = function() {
    var out;
    
    out = new ListeValeurs();

    if (this.s > 0)
    {
        this.s -= this.ds;
        this.x1 = -this.s * Math.sin(this.i1 / 180. * Math.PI);
        this.y1 =  this.s * Math.cos(this.i1 / 180. * Math.PI);
        this.x2 = 0;
        this.y2 = 0;
    }
    else if (this.s <= 0 && this.s > -8)
    {
        
        this.s -=  this.ds;
        
        this.x1 =  this.s * Math.sin(this.r1 / 180. * Math.PI);
        this.y1 = -this.s * Math.cos(this.r1 / 180. * Math.PI);
        
        this.x2 = -this.s * Math.sin(this.i2 / 180. * Math.PI);
        this.y2 =  this.s * Math.cos(this.i2 / 180. * Math.PI) * (Math.abs(this.i1) <= 90 ? 1. : -1.);
        
    }
    else
        return null;


    out.ajouterDouble('x1', this.x1);
    out.ajouterDouble('y1', this.y1);
    out.ajouterDouble('x2', this.x2);
    out.ajouterDouble('y2', this.y2);
    out.ajouterDouble('i1', this.i1);
    out.ajouterDouble('r1', this.r1);
    out.ajouterDouble('i2', this.i2);
    out.ajouterDouble('i1b', (this.s1 == 'on' ? Math.sin(this.i1 / 180 * Math.PI) : this.i1));
    out.ajouterDouble('i2b' , (this.s2 == 'on' ? Math.sin(this.i2 / 180 * Math.PI) : this.i2));

    return out;
}

    // - - - III.2 Calcul complet d'un coup (utilisé avec les affichages "plot" et "tableau") - - - - 
SnellDescartes.prototype.calcul = function(lin) {
    var out;
    
    out = new ListeValeurs();

    // - 3.1.1.1 Chargement des variables d'entrée -
    this.n1 = lin.lireDouble('n1');
    this.n2 = lin.lireDouble('n2');
    this.s1 = lin.lireString('s1');
    this.s2 = lin.lireString('s2');


    for (this.i1 = 0; this.i1 <= 90 ; this.i1 = this.i1 + 0.1)
    {
        this.r1 = -this.i1;
        this.i2 = 180 / Math.PI * Math.asin(this.n1 / this.n2 * Math.sin(this.i1 / 180 * Math.PI));
        
        out.ajouterDouble('x1', this.x1);
        out.ajouterDouble('y1', this.y1);
        out.ajouterDouble('x2', this.x2);
        out.ajouterDouble('y2', this.y2);
        out.ajouterDouble('i1', this.i1);
        out.ajouterDouble('i2', this.i1);
        out.ajouterDouble('r1', this.r1);
        out.ajouterDouble('i1b', (this.s1 == 'on' ? Math.sin(this.i1 / 180 * Math.PI) : this.i1));
        out.ajouterDouble('i2b' , (this.s2 == 'on' ? Math.sin(this.i2 / 180 * Math.PI) : this.i2));
    }
    return out;
}
